#include <stdlib.h>
#include <stdio.h>
void MvdSetHmBits( char cline[80], int hmbit[16], int amubit[8][33] )
{
   /* scan a command line to look for definitions of HM bits         */

   /* input cline, hmbit                                             */

   /* command  bits used   purpose                                   */
   /* _______  _________   __________                                */
   /* lvl1     hmbit[0-3]  adjusts the assumed level-1 delay from 33 */
   /*                      to 47 beam clock ticks.                   */
   /* --       hmbit[4]    not used                                  */
   /* prepost  hmbit[5]    sets the separation between pre and post  */
   /*                      samples (in the amu cells) to either 1    */
   /*                      (bit=0) or 2 (bit=1) beam clock ticks.    */
   /* phase    hmbit[6-7]  level-1 phase adjust, for fine adjustment */
   /*                      of level-1 delay. An integer from 1 to 3. */
   /*                      = 0 -->  0.00 clocks --> hmbit[6,7]=0,0   */
   /*                      = 1 --> -0.25 clocks --> hmbit[6,7]=1,0   */
   /*                      = 2 --> -0.50 clocks --> hmbit[6,7]=0,1   */
   /*                      = 3 --> -0.75 clocks --> hmbit[6,7]=1,1   */
   /* test     hmbit[8]    = 1 turns on "test mode" with ADCs always */
   /*                          returned as 0,1,2,...                 */
   /*                      = 0 is normal running mode                */
   /* ren      hmbit[9]    ren=1 enables internally driven preamp    */
   /*                      resets, with frequency controlled by ir0  */
   /*                      and ir1 bits                              */
   /* --       hmbit[10-11] these bits are used to set the number of */
   /*                      bits used to digitize the ADCs. Because   */
   /*                      the MVD does not expect to ever use       */
   /*                      anything but 10 bits, no command was      */
   /*                      written to change these bits. They should */
   /*                      always be hmbit[10]=1, [11]=0.            */
   /* raw      hmbit[12]   =1 Set "raw" mode (2 packets per event)   */
   /*                      =0 correlator mode, normal running setting*/
   /*                         This command also sets the related amu */
   /*                         serial string to the correct values    */
   /* calen    hmbit[13]   =1 bench cal enabled (specific channels   */
   /*                         still need to be enabled)              */
   /*                      =0 bench cal disabled                     */
   /* renfreq  hmbit[14-15]=0 means highest reset frequency          */
   /*                      =3 means lowest reset frequency           */
   /*                      has no effect unless iren=1               */

   /* Author = JPSullivan 23-May-2000 (based on labwindows code      */
   /*          originally written by BSchlei and SSRyu)              */

   /* JPS modified 1-Aug-2000: added more commands                   */

   /* **********************************************************     */

   void MvdSetLvl1Bits ( char cline[80], int hmbit[16] );
   void MvdSetPrePost ( char cline[80], int hmbit[16] );
   void MvdSetPhase ( char cline[80], int hmbit[16] );
   void MvdTestMode ( char cline[80], int hmbit[16] );
   void MvdSetRaw ( char cline[80], int hmbit[16], int amubit[8][33] );
   void MvdSetRen ( char cline[80], int hmbit[16] );
   void MvdSetCalEn ( char cline[80], int hmbit[16] );
   void MvdSetRenFreq ( char cline[80], int hmbit[16] );

   int i;
   char arg1[80];
   char arg2[80];
   char Cmd_lvl1[5]={'l','v','l','1'};
   char Cmd_prepost[8]={'p','r','e','p','o','s','t'};
   char Cmd_phase[6]={'p','h','a','s','e'};
   char Cmd_testmode[5]={'t','e','s','t'};
   char Cmd_rawmode[4]={'r','a','w'};
   char Cmd_setren[4]={'r','e','n'};
   char Cmd_calen[6]={'c','a','l','e','n'};
   char Cmd_renfreq[8]={'r','e','n','f','r','e','q'};

   /* ********************************************************** */

   sscanf (cline, "%s%s", arg1,arg2 );

   for ( i=0; i<strlen(arg1); i++ )
      arg1[i] = tolower(arg1[i]);

   if ( arg1[0] != 'h' || arg1[1] != 'm' ) 
      return;

   if ( strcmp ( Cmd_lvl1, arg2 ) == 0 ) 
   {
     MvdSetLvl1Bits ( cline, hmbit );
   }
   else if ( strcmp ( Cmd_prepost, arg2 ) == 0 ) 
   {
     MvdSetPrePost ( cline, hmbit );
   }
   else if ( strcmp ( Cmd_phase, arg2 ) == 0 ) 
   {
     MvdSetPhase ( cline, hmbit );
   }
   else if ( strcmp ( Cmd_testmode, arg2 ) == 0 ) 
   {
     MvdTestMode ( cline, hmbit );
   }
   else if ( strcmp ( Cmd_setren, arg2 ) == 0 ) 
   {
     MvdSetRen ( cline, hmbit );
   }
   else if ( strcmp ( Cmd_rawmode, arg2 ) == 0 ) 
   {
     MvdRawMode ( cline, hmbit, amubit );
   }
   else if ( strcmp ( Cmd_calen, arg2 ) == 0 ) 
   {
     MvdSetCalEn ( cline, hmbit );
   }
   else if ( strcmp ( Cmd_renfreq, arg2 ) == 0 ) 
   {
     MvdSetRenFreq ( cline, hmbit );
   }
   else
   {
     fprintf(stderr,"MvdSetHmBits: unknow command=%s\n",arg2);
   }

   return;
}
