#include <stdlib.h>
#include <stdio.h>
void MvdShowAmuBits( int amubit[8][33] )
{
   /* Show current AMU/ADC serial string setup                       */

   /* input amubit                                                   */

   /* Author = JPSullivan  8-May-2001                                */

   /* Modified:                                                      */

   /* **********************************************************     */

   float MvdAmuDacVolts ( int amubit[8][33], int iamu, int bit1, 
                          int bitlast, float vmin, float vmax,
                          float dacmax );

   int MvdAmuDacInt ( int amubit[8][33], int iamu, int bit1, 
                        int bitlast );

   int i;                /* loop index (used for AMU chip number=0,7)*/
   float vref;           /* vref  value in volts                     */
   float vcorr;          /* vcorr value in volts                     */
   float iref;           /* iref  value in microamps                 */
   int amuspychan;       /* channel number for AMU spy line          */
   char AmuMuxonoff[4];  /* 'ON' or 'OFF' for AMU MUX (spy line)     */
   int tgvspychan;       /* channel number for TGV (preamp) spy line */
   char TgvMuxonoff[4];  /* 'ON' or 'OFF' for TGV MUX (spy line)     */
   char Corronoff[4];    /* 'ON' or 'OFF' for correlator             */

   float DACmax = 63.;   /* all dacs are 6 bits, so they count to 63 */
 
   /* Vref DAC is from 0.2 to 4.80 Volts, using bits 7-12 */
   float AMUVREFmin =  0.20;
   float AMUVREFmax =  4.80;
   int   AMUVREFbit1=     7;
   int   AMUVREFbitlast= 12;
 
   /* Vcorr DAC is from 0.10 to 2.50 Volts, using bits 13-18 */
   float AMUVCORmin =  0.10;
   float AMUVCORmax =  2.50;
   int   AMUVCORbit1=    13;
   int   AMUVCORbitlast= 18;
 
   /* Iref DAC is from 2.0 to 70.0 Volts */
   float AMUIREFmin =  70.0;
   float AMUIREFmax =   2.0;
   int   AMUIREFbit1=    19;
   int   AMUIREFbitlast= 24;
 
   /* AMU MUX (spy line) channel is specified by bits 1-5 */
   int   AMUMUXbit1=    1;
   int   AMUMUXbitlast= 5;
 
   /* TGV MUX (spy line) channel is specified by bits 1-5 */
   int   TGVMUXbit1=    28;
   int   TGVMUXbitlast= 32;
 
   /* ********************************************************** */

   /* header for table which follows: */
   printf ( " _____________________________________________________________________\n");
   printf ( " AMU/ADC serial string parameters:\n");
   printf ( "  chip  correlator   Vref  Vcorr   Iref    AMU spy        preamp spy \n");
   printf ( "  num.    [on/off]    [V]    [V]   [uA]    on/off  chan   on/off  chan\n");
   printf ( " _____  __________  _____  _____  _____    ____________   ____________\n");

   /* ******************************** */
   /* loop is over i=0,7 AMU/ADC chips */

   for (i=0; i<8; i++)
   {
      vref = MvdAmuDacVolts ( amubit, i, AMUVREFbit1, AMUVREFbitlast,
                              AMUVREFmin, AMUVREFmax, DACmax );
      vcorr= MvdAmuDacVolts ( amubit, i, AMUVCORbit1, AMUVCORbitlast,
                              AMUVCORmin, AMUVCORmax, DACmax );
      iref = MvdAmuDacVolts ( amubit, i, AMUIREFbit1, AMUIREFbitlast,
                              AMUIREFmin, AMUIREFmax, DACmax );

      /* check whether AMU MUX is on or off, or set inconsistently */
      if ( amubit[i][0]==1 && amubit[i][25]==1 )
      {
        AmuMuxonoff[0]=' ';
        AmuMuxonoff[1]='O';
        AmuMuxonoff[2]='N';
        AmuMuxonoff[3]=  0;
      }
      else if ( amubit[i][0]==0 && amubit[i][25]==0 )
      {
        AmuMuxonoff[0]='O';
        AmuMuxonoff[1]='F';
        AmuMuxonoff[2]='F';
        AmuMuxonoff[3]=  0;
      }
      else
      {
        /* One bit on, one off, should not happen, state is unknown */
        AmuMuxonoff[0]=' ';
        AmuMuxonoff[1]='?';
        AmuMuxonoff[2]=' ';
        AmuMuxonoff[3]=  0;
      }
      amuspychan = MvdAmuDacInt ( amubit, i, AMUMUXbit1, AMUMUXbitlast );

/*    TGV spy channel:    */

      if ( amubit[i][26]==1 && amubit[i][27]==1 )
      {
        TgvMuxonoff[0]=' ';
        TgvMuxonoff[1]='O';
        TgvMuxonoff[2]='N';
        TgvMuxonoff[3]=  0;
      }
      else if ( amubit[i][26]==0 && amubit[i][27]==0 )
      {
        TgvMuxonoff[0]='O';
        TgvMuxonoff[1]='F';
        TgvMuxonoff[2]='F';
        TgvMuxonoff[3]=  0;
      }
      else
      {
        /* One bit on, one off, should not happen, state is unknown */
        TgvMuxonoff[0]=' ';
        TgvMuxonoff[1]='?';
        TgvMuxonoff[2]=' ';
        TgvMuxonoff[3]=  0;
      }
      tgvspychan = MvdAmuDacInt ( amubit, i, TGVMUXbit1, TGVMUXbitlast );

      /* Check whether correlator is on or off */

      if ( amubit[i][6]==1 )
      {
        Corronoff[0]=' ';
        Corronoff[1]='O';
        Corronoff[2]='N';
        Corronoff[3]=  0;
      }
      else
      {
        Corronoff[0]='O';
        Corronoff[1]='F';
        Corronoff[2]='F';
        Corronoff[3]=  0;
      }
      printf ( "  %4d      %s     %5.2f  %5.2f  %5.2f    %s  %7d   %s  %7d\n",
              i,Corronoff, vref, vcorr, iref, AmuMuxonoff, amuspychan, TgvMuxonoff, tgvspychan);

   }

   return;
}
float MvdAmuDacVolts ( int amubit[8][33], int iamu, int bit1, int bitlast,
                       float vmin, float vmax, float dacmax )
{
   /* Translate bit1 --> bitlast bits out of the serial string  */
   /* amubit for chip iamu into a value in volts. vmin and vmax */
   /* are the min and max values of the DAC in volts and dacmax */
   /* is the maximum value which can be specified by the DAC.   */

   /* input amubit = serial string bits for all 8 AMU/ADC chips      */
   /*       iamu   = chip number (1st index of amubit)               */
   /*       bit1   = 1st bit to convert (LSB)                        */
   /*       bitlast= last bit to convert (MSB)                       */
   /*       vmin   = voltage corresponds to DAC value = 0            */
   /*       vmax   = voltage value corresponding to DAC=dacmax       */
   /*       dacmax = max value the DAC can count to (always 63 here) */

   /* Author = JPSullivan  8-May-2001                                */

   /* Modified:                                                      */

   /* **********************************************************     */

   int j;        /* for loop over bits       */
   int ivolt;    /* DAC value, integer       */
   int ipower2;  /* to caclulate powers of 2 */
   float volt;   /* DAC value in volts       */

   /* ********************************************************** */

   ivolt = 0;
   ipower2=1;
   for (j=bit1; j<=bitlast; j++) {
     ivolt = ivolt + amubit[iamu][j]*ipower2;
     ipower2*=2;
   }
   volt = ivolt/dacmax;
   volt *= (vmax-vmin);
   volt += vmin;

   return volt;
}
int MvdAmuDacInt ( int amubit[8][33], int iamu, int bit1, int bitlast)
{
   /* Translate bit1 --> bitlast bits out of the serial string  */
   /* amubit for chip iamu into an integer value (returned)     */

   /* input amubit = serial string bits for all 8 AMU/ADC chips      */
   /*       iamu   = chip number (1st index of amubit)               */
   /*       bit1   = 1st bit to convert (LSB)                        */
   /*       bitlast= last bit to convert (MSB)                       */

   /* Author = JPSullivan  8-May-2001                                */

   /* Modified:                                                      */

   /* **********************************************************     */

   int j;       /* for loop over bits */
   int idata;   /* DAC value, integer */
   int ipower2; /* for powers of 2    */

   /* ********************************************************** */

   idata = 0;
   ipower2=1;
   for (j=bit1; j<=bitlast; j++) {
     idata = idata + amubit[iamu][j]*ipower2;
     ipower2*=2;
   }

   return idata;
}
