#include <stdlib.h>
#include <stdio.h>
void MvdShowHmBits( int hmbit[16] )
{
   /* Show current HM serial string setup                            */

   /* input hmbit                                                    */

   /* Author = JPSullivan  8-May-2001                                */

   /* Modified:                                                      */

   /* **********************************************************     */

   int MvdHmDacInt ( int hmbit[16], int bit1, int bitlast );

   int i;

   int level_1_delay;  /* level 1 delay in beam clock ticks (33-47) */
   int Level1_1=0;     /* first bit to specify level 1 delay        */
   int Level1_last=3;  /* last bit to specify level-1 delay         */

   int pre_post_sep;   /* separation (beam clocks) between pre/post */

   int iphase;         /* phase adjust in 1/4 beam clock units      */
   float phase;        /* phase adjust (with sign) in beam clocks   */
   int Phase_1=6;      /* first bit to specify phase adjust         */
   int Phase_last=7;   /* last bit to specify phase adjust          */

   char TestMode[4];   /* specifies test mode on or off             */
   char InternalResetMode[4];  /* internal preamp Reset mode on/off */

   int nadcbits;       /* number of ADC bits (should be always 10) */
   int AdcBits_1=10;   /* first bit to specify phase adjust        */
   int AdcBits_last=11;/* last bit to specify phase adjust         */

   char RawMode[4];    /* specifies raw mode on or off             */
   char CalMode[4];    /* specifies calibration mode on or off     */

   int reset_freq;     /* reset frequency divider                  */
   int ResetFreq_1=14; /* 1st bit for reset frequency divider      */
   int ResetFreq_last=15;  /* last bit for reset frequency divider */

   /* ************************************************************ */

   printf ( " _____________________________________________________________________\n");
   printf ( " HM serial string parameters:\n");

   level_1_delay = MvdHmDacInt ( hmbit, Level1_1, Level1_last );
   level_1_delay+=32;

   pre_post_sep = hmbit[5]+1;   /* pre/post separation */

   iphase =  MvdHmDacInt ( hmbit, Phase_1, Phase_last );
   phase = -0.25*(float)iphase;

   if ( hmbit[8] == 1 )
   {
      TestMode[0]='O';
      TestMode[1]='N';
      TestMode[2]=' ';
      TestMode[3]= 0;
   }
   else
   {
      TestMode[0]='O';
      TestMode[1]='F';
      TestMode[2]='F';
      TestMode[3]= 0;
   }

   if ( hmbit[9] == 1 )
   {
      InternalResetMode[0]='O';
      InternalResetMode[1]='N';
      InternalResetMode[2]=' ';
      InternalResetMode[3]= 0;
   }
   else
   {
      InternalResetMode[0]='O';
      InternalResetMode[1]='F';
      InternalResetMode[2]='F';
      InternalResetMode[3]= 0;
   }

   nadcbits =  MvdHmDacInt ( hmbit, AdcBits_1, AdcBits_last );
   nadcbits +=9;

   if ( hmbit[12] == 1 )
   {
      RawMode[0]='O';
      RawMode[1]='N';
      RawMode[2]=' ';
      RawMode[3]= 0;
   }
   else
   {
      RawMode[0]='O';
      RawMode[1]='F';
      RawMode[2]='F';
      RawMode[3]= 0;
   }

   if ( hmbit[13] == 1 )
   {
      CalMode[0]='O';
      CalMode[1]='N';
      CalMode[2]=' ';
      CalMode[3]= 0;
   }
   else
   {
      CalMode[0]='O';
      CalMode[1]='F';
      CalMode[2]='F';
      CalMode[3]= 0;
   }

   reset_freq =  MvdHmDacInt ( hmbit, ResetFreq_1, ResetFreq_last );

   printf ( " (Level 1 delay)-phase= %2d%4.2f beam clocks",
            level_1_delay, phase);
   printf ( "     pre/post separation= %d\n",pre_post_sep);
   printf ( " Raw mode= %s",RawMode);
   printf ( "      Cal mode= %s",CalMode);
   printf ( "      Test mode= %s",TestMode);
   printf ( "      %d bit ADCs\n",nadcbits);
   printf ( " Internal preamp resets= %s",InternalResetMode);
   printf ( "               Reset frequency divider = %d\n",reset_freq);
   
   return;
}
int MvdHmDacInt ( int hmbit[16], int bit1, int bitlast)
{
   /* Translate bit1 --> bitlast bits out of the serial string  */
   /* hmbit into an integer value (returned)                    */

   /* input hmbit  = serial string bits Heap Manager            */
   /*       bit1   = 1st bit to convert (LSB)                   */
   /*       bitlast= last bit to convert (MSB)                  */

   /* Author = JPSullivan  8-May-2001                           */

   /* Modified:                                                 */

   /* ********************************************************* */

   int j;       /* for loop over bits */
   int idata;   /* DAC value, integer */
   int ipower2; /* for powers of 2    */

   /* ********************************************************* */

   idata = 0;
   ipower2=1;
   for (j=bit1; j<=bitlast; j++) {
     idata = idata + hmbit[j]*ipower2;
     ipower2*=2;
   }

   return idata;
}
