#include <stdlib.h>
#include <stdio.h>

int main (int argv, char *argc[])
{
   /* Read Serial string data from a file in the format used    */
   /* by the arcnet program, read any commands to change the    */
   /* data from standard input,  and write a new file with the  */
   /* changes.                                                  */

   /* Commands are summarized in comments in MvdSetHmBits.c,    */
   /* MvdSetAmuBits.c, MvdSetTgvBits.c                          */

   /* input file is first arguement (required)                  */
   /* output file is 2nd arguement (required)                   */

   /* Author = JPSullivan 23-May-2000 (based on labwindows code */
   /*          originally written by BSchlei and SSRyu)         */
   /* Mofications: 4-Aug-2000 JPS cleanup and a few comments    */
   /*              7-May-2001 JPS add status output at begin    */
   /*             21-Jun-2001 JPS comment out status outout     */

   /* ********************************************************* */

   FILE *fpin;
   FILE *fpout;
   FILE *fprsp;

   void MvdReadMcmSerial2  ( FILE *fpin, int SerialBitsIn[1800] );
   void MvdUnpackMcmSerial2( int SerialBitsIn[1800], int hmbit[16],
                             int tgvbit[8][190], int amubit[8][33] );
   void MvdPackMcmSerial2  ( int hmbit[16], int tgvbit[8][190], 
                             int amubit[8][33], int SerialBitsOut[1800] );
   void MvdWriteMcmSerial2( FILE *fpout, int SerialBitsOut[1800] );
   void MvdWriteMcmResponse( FILE *fprsp, int SerialBitsOut[1800] );
   void MvdGetSerialChanges( int hmbit[16], int tgvbit[8][190], 
                             int amubit[8][33] );
   void MvdShowStatus      ( int hmbit[16], int tgvbit[8][190], 
                             int amubit[8][33] );
									
   int hmbit[16];       /* heap manager serial bits */
   int amubit[8][33];   /* AMU/ADC serial bits      */
   int tgvbit[8][190];  /* TGV serial bits          */

   int SerialBitsIn[1800];   /* Serial data input from data file (chaintest format)  */
   int SerialBitsOut[1800];  /* Serial data reorganized to be in order bits are sent */

   /* ********************************************************** */

   /* start by checking for the required number of arguements and */
   /* opening both the input and output files */

   if( argv < 3 )
   {
      fprintf(stderr,"\nProper usage is: " );
      fprintf(stderr,"\nserial filein fileout \n" );
      exit(1);
   }

   if (( fpin = fopen( argc[1], "r" )) == NULL )
   {
      fprintf(stderr, "Error opening input file, %s!", argc[1]);
      exit(1);
   }

   if (( fpout = fopen( argc[2], "w" )) == NULL )
   {
      fprintf(stderr, "Error opening output file, %s!", argc[2]);
      exit(1);
   }

   if (( fprsp = fopen( "serial_response.rsp", "w" )) == NULL )
   {
      fprintf(stderr, "Error opening response output file, !");
      exit(1);
   }


   /* ****************** */
   /* read input, upack: */

   MvdReadMcmSerial2( fpin, SerialBitsIn );

   MvdUnpackMcmSerial2( SerialBitsIn, hmbit, tgvbit, amubit);

   /* ***********************************      */
   /* print current bit settings               */
   /* MvdShowStatus ( hmbit, tgvbit, amubit ); */

   /* ***********************************      */
   /* allow some parameters to be changed --   */
   /* This is the largest part of this program */

   MvdGetSerialChanges( hmbit, tgvbit, amubit );

   /* ***************************************************************** */
   /* at the end, repack the serial string and write to the output file */

   MvdPackMcmSerial2( hmbit, tgvbit, amubit, SerialBitsOut );
   MvdWriteMcmSerial2( fpout, SerialBitsOut );
   MvdWriteMcmResponse( fprsp, SerialBitsOut );

   return;
}
